<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))) . '/includes/404.php';
}

/**
 * Frame_admin
 *
 * Сохранение элемента
 */
class Frame_admin
{

	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var string таблица в базе данных
	 */
	public $table;

	/**
	 * @var array настройки отображения
	 */
	public $config_module = array ();

	/**
	 * @var array поля таблицы с типами
	 */
	public $variable_table = array ();

	/**
	 * @var array переводимые поля
	 */
	public $language_names = array ();

	/**
	 * @var array массив полей таблицы для SQL-запроса на обновление
	 */
	public $query;

	/**
	 * @var array массив новых значений для SQL-запроса на обновление
	 */
	public $value;

	/**
	 * @var array массив старых значений обновляемых полей
	 */
	public $oldrow = array ();

	/**
	 * @var integer номер страницы, к которому прикреплен текущий элемент
	 */
	public $site_id;

    /**
     * @var integer номер родителя, к которому прикреплен текущий элемент
     */
	public $parent_id;

    /**
     * @var integer номер каталога, к которому прикреплен текущий элемент
     */

	public $cat_id;

	/**
	 * @var integer порядковый номер элемента
	 */
	public $sort;

    /**
     * Конструктор класса
     *
     * @param Initadmin $diafan
     * @return \Frame_admin
     */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	public function is_variable($key)
	{
		$res = FALSE;
		foreach ($this->variable_table as $title => $arr)
		{
			if (array_key_exists($key, $arr))
			{
				$res = TRUE;
				break;
			}
		}
		return $res;
	}

	/**
	 * Добавляет элемент в базу данных
	 *
	 * @return boolean
	 */
	public function new_module()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if (!$this->diafan->_user->checked)
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		// Проверяет права на добавление
		if ($this->diafan->admin_roles('edit'))
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		// Проверяет, пришел ли запрос из формы добавления/редактирования
		if (empty( $_POST["save_post"] ))
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		// Прошел ли пользователь проверку идентификационного хэша
		if (!$this->diafan->_user->checked)
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		$def = array ();
		if ($this->config_module["parent"])
		{
			$def['parent_id'] = "'" . $this->diafan->parent . "'";
		}
		if ($this->config_module["element_site"])
		{
			$def['site_id'] = "'" . intval(preg_replace('/\D/', '', $_POST["site_id"])) . "'";
		}

		DB::query("INSERT INTO {" . $this->table . "} (" . implode(',', array_keys($def)) . ") VALUES (" . implode(',', $def) . ")");

		$this->diafan->save = DB::last_id($this->table);

		if (!$this->diafan->save)
		{
			echo '<script language="javascript" type="text/javascript">alert(\'' . sprintf(_ADMIN_ERROR_INSERT_INTO, DB_PREFIX . $this->table) . '\');history.go(-1);</script>';
			return FALSE;
		}

		$this->save_module();
		return TRUE;
	}

	/**
	 * Сохраняет изменения
	 *
	 * @return boolean
	 */
	public function save_module()
	{
		// Проверяет, пришел ли запрос из формы добавления/редактирования
		if (empty( $_POST["save_post"] ))
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		// Прошел ли пользователь проверку идентификационного хэша
		if (!$this->diafan->_user->checked)
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		// Проверка прав на сохранение
		if ($this->diafan->admin_roles('edit') && !$this->diafan->savenew)
		{
			$this->diafan->redirect(URL);
			return FALSE;
		}

		// Массив $this->oldrow содержит прежние значения полей для сохраняемого элемента
		if (!$this->config_module["config"])
		{
			$this->oldrow = DB::fetch_array(DB::query("SELECT * FROM {" . $this->table . "} WHERE id = %d" . ( $this->config_module["trash"] ? " AND trash='0'" : '' ), $this->diafan->save));
			if (!$this->oldrow)
			{
				$this->diafan->redirect(URL);
				return FALSE;
			}
		}

		// Подготовка значений полей для элемента в соответсвии с указанными типами

		foreach ($this->variable_table as $title => $variable_table)
		{
			$this->prepare_new_values($variable_table);
		}


		// Сохраняет конфигурацию модуля
		if ($this->config_module["config"])
		{
			for ($q = 0; $q < count($this->value); $q++)
			{
				$this->value[$q] = str_replace("\n", '', $this->value[$q]);
			}
			DB::query("DELETE FROM {config} WHERE module_name='%s' AND site_id='%d'" . ( $this->language_names ? " AND (lang_id='" . _LANG . "' OR name NOT IN ('" . implode("','", $this->language_names) . "'))" : "" ), $this->diafan->module, $this->diafan->site);
			for ($q = 0; $q < count($this->value); $q++)
			{
				list( $name, $mask ) = explode('=', $this->query[$q]);
				$name = str_replace('`', '', $name);

				// записываем значение в конфигурацю если оно не пустое или если конфигурация сохраняется для раздела и оно отличается от основной конфигурации
				if (!$this->diafan->site && $this->value[$q] || $this->diafan->site && DB::query_result("SELECT value FROM {config} WHERE module_name='%s' AND site_id='0' AND name='%h'" . ( !empty( $this->language_names[$name] ) ? " AND lang_id='" . _LANG . "'" : '' ), $this->diafan->module, $name) != $this->value[$q])
				{
					DB::query("INSERT INTO {config} (name, module_name, value, site_id, lang_id) VALUES ('%h', '%h', " . $mask . ", '%d', '%d')", $name, $this->diafan->module, $this->value[$q], $this->diafan->site, in_array($name, $this->language_names) ? _LANG : '');
				}
			}
			$this->diafan->done = 1;

			// Удаляет кэш конфигурации модулей
			$this->diafan->_cache->delete("configmodules", "site");
		}

		// Сохраняет элемент
		else
		{
			if (!DB::query("UPDATE {" . $this->table . "} SET " . implode(', ', $this->query) . " WHERE id = %d", array_merge($this->value, array ( $this->diafan->save ))))
			{
				return FALSE;
			}

			if ($this->config_module['order'] && empty( $this->oldrow['sort'] ) && !DB::query("UPDATE {" . $this->table . "} SET sort=id WHERE id=%d", $this->diafan->save))
			{
				return FALSE;
			}
		}

		// Удаляет кэш модуля
		//echo $this->diafan->module;exit;
		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->save_redirect();
		return TRUE;
	}

	/**
	 * Подготавливает новые значения для сохранения
	 *
	 * @return boolean TRUE
	 */
	private function prepare_new_values($variable_table)
	{
		foreach ($variable_table as $key => $type_value)
		{
			$func = 'save_' . str_replace('-', '_', $key) . '_module' . ( $this->config_module["config"] ? '_config' : '' );
			$name = $key . ( in_array($key, $this->language_names) ? _LANG : '' );

			if (method_exists($this, $func))
			{
				$this->$func();
			}
			elseif ($type_value == 'module')
			{
				if (file_exists(ABSOLUTE_PATH . 'modules/' . $key . '/admin/' . $key . '.admin.save.php'))
				{
					Customization::inc('modules/' . $key . '/admin/' . $key . '.admin.save.php');
					$func = 'save' . ( $this->config_module["config"] ? '_config' : '' );
					$class = ucfirst($key) . '_admin_save';
					if (method_exists($class, $func))
					{
						eval( $class . '::' . $func . '($this);' );
					}
				}
			}
			elseif ($type_value == 'password')
			{
				if (!empty( $_POST[$key] ))
				{
					$this->query[] = $name . "='%s'";
					$this->value[] = code_password($_POST[$key]);
				}
			}
			elseif ($type_value == 'date' || $type_value == 'datetime')
			{
				$this->query[] = $name . "='%d'";
				$this->value[] = $this->diafan->unixdate($_POST[$key]);
			}
			elseif ($type_value == 'hr' || $type_value == 'disabled')
			{
				continue;
			}
			elseif ($type_value == 'floattext')
			{
				$this->value[] = str_replace(',', '.', !empty( $_POST[$key] ) ? $_POST[$key] : 0);
				$this->query[] = $name . "='%f'";
			}
			else
			{
				if ($type_value == 'editor')
				{
					$text = !empty( $_POST[$key] ) ? $_POST[$key] : '';
					if (!empty( $_POST[$key . "_typograf"] ))
					{
						include_once( ABSOLUTE_PATH . "plugins/remotetypograf.php" );

						$remoteTypograf = new RemoteTypograf();

						$remoteTypograf->htmlEntities();
						$remoteTypograf->br(false);
						$remoteTypograf->p(true);
						$remoteTypograf->nobr(3);
						$remoteTypograf->quotA('laquo raquo');
						$remoteTypograf->quotB('bdquo ldquo');

						$text = $remoteTypograf->processText($text);
					}
					$this->value[] = $text;
				}
				elseif ($type_value == 'textarea')
				{
					$this->value[] = !empty($_POST[$key]) ? nl2br(htmlspecialchars($_POST[$key])) : '';
				}
				else
				{
					$this->value[] = !empty( $_POST[$key] ) ? $_POST[$key] : '';
				}

				if ($type_value == 'text' || $type_value == 'select')
				{
					$this->query[] = $name . "='%h'";
				}
				elseif ($type_value == 'checkbox' || $type_value == 'numtext')
				{
					$this->query[] = $name . "='%d'";
				}
				else //editor,textarea,none,function,...
				{
					$this->query[] = $name . "='%s'";
				}
			}
		}
		return TRUE;
	}

	/**
	 * Производит перенаправление на страницу редактирования, на список и пр.
	 *
	 * @return boolean TRUE
	 */
	protected function save_redirect()
	{
		// если сохраняли Ajaxом, отдаем положительный результат
		if (!empty( $_SERVER["HTTP_X_REQUESTED_WITH"] ) && $_SERVER["HTTP_X_REQUESTED_WITH"] == 'XMLHttpRequest')
		{
			echo '{success:TRUE}';
			exit;
		}

		$status=( $this->diafan->err ? 'error' . $this->diafan->err . '/' : '' ) . ( $this->diafan->done ? 'success' . $this->diafan->done . '/' : '' );


		// Для модуля Языки сайта
		if ($this->table == "language")
		{
			if (_LANG == $this->diafan->save)
			{
				$this->diafan->redirect(BASE_PATH . ADMIN_FOLDER . '/language/');
			}
			else
			{
				$this->diafan->redirect(URL . $status);
			}
		}

		$parent='';
		if(!empty($this->parent_id))
		{
			$parent='parent'.$this->parent_id.'/';
		}

		$cat='';
		if(!empty($this->cat_id))
		{
			$cat='cat'.$this->cat_id.'/';
		}

		// "Применить" ($_POST["noOut"]) - редирект на edit_module
		if ($_POST["noOut"])
		{
			switch ($_POST["noOut"])
			{
				case '2':
					$this->diafan->redirect(URL .$cat. $parent . 'addnew1');
					break;
				default:
					$this->diafan->redirect(URL .$cat. $parent . $status . 'edit' . $this->diafan->save . '/');
			}
		}

		// Если к странице прикреплен модуль, то редирект на модуль
		$addmodule = !empty( $_POST["addmodule"] ) ? preg_replace('/[^A-Za-z-]+/', '', $_POST["addmodule"]) : '';
		if ($this->diafan->rewrite == "site" && $addmodule && $addmodule != "reminding" && file_exists(ABSOLUTE_PATH . 'modules/' . $addmodule . '/admin/' . $addmodule . '.admin.php'))
		{
			$this->diafan->redirect(BASE_PATH_HREF . $addmodule . '/site' . $this->diafan->save . '/' .$cat. $parent . $status);
			return TRUE;
		}

		if(DB::query_result("SELECT COUNT(*) FROM {%h} WHERE trash='0'", $this->table) > 10)
		{
			$ankor = '#' . $this->diafan->save;
		}

		// "Сохранить и выйти" - редирект на show_module
		$this->diafan->redirect(URL .$cat. $parent .$status . $ankor);
		return TRUE;
	}

	//----------------------------------------------------------------------//
	// Функции сохранения полей

	/**
	 * Сохранение поля "Категория"
	 *
	 * @return boolean
	 */
	protected function save_cat_id_module()
	{
		$_POST['cat_id']=(int)preg_replace('/\D/','',$_POST['cat_id']);

		$this->query[] = "cat_id=%d";
		$this->value[] = $_POST["cat_id"];

		$this->cat_id = $_POST['cat_id'];

		if ($this->config_module["element_multiple"])
		{
			DB::query("DELETE FROM {%s_category_rel} WHERE element_id=%d", $this->diafan->module, $this->diafan->save);
			DB::query("INSERT INTO {%s_category_rel} (element_id, cat_id) VALUES('%d', '%d')", $this->diafan->module, $this->diafan->save, $_POST["cat_id"]);

			if (!empty( $_POST["cat_ids"] ) && ! empty($_POST["user_additional_cat_id"]) && is_array($_POST["cat_ids"]))
			{
				foreach ($_POST["cat_ids"] as $cat_id)
				{
					if ($cat_id != $_POST["cat_id"])
					{
						DB::query("INSERT INTO {%s_category_rel} (element_id, cat_id) VALUES('%d', '%d')", $this->diafan->module, $this->diafan->save, $cat_id);
					}
				}
			}
		}
		elseif ($this->config_module["parent"] && !$this->diafan->savenew && DB::query_result("SELECT cat_id FROM {%h} WHERE id=%d", $this->table, $this->diafan->save) != $_POST["cat_id"])
		{
			$children = $this->diafan->get_children($this->diafan->save, $this->table);
			if ($children)
			{
				DB::query("UPDATE {%h} SET cat_id=%d WHERE id IN (%h)", $this->table, $_POST["cat_id"], implode(",", $children));
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Страница"
	 *
	 * @return boolean TRUE
	 */
	private function save_site_id_module()
	{
		$this->query[] = "site_id='%d'";
		if (!$this->site_id)
		{
			$this->get_site_id();
		}
		$this->value[] = $this->site_id;
		if ($this->config_module["parent"] && ! $this->diafan->savenew)
		{
			$children = $this->diafan->get_children($this->diafan->save, $this->table);
			if($children)
			{
				DB::query("UPDATE {".$this->table."} SET site_id=%d WHERE id IN (%h)", $this->site_id, implode(",", $children));
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Страница"
	 *
	 * @return boolean TRUE
	 */
	public function get_site_id()
	{
		$this->site_id = !empty( $_POST["site_id"] ) ? $_POST["site_id"] : 0;
		if ($this->config_module["element"])
		{
			$this->site_id = DB::query_result("SELECT site_id FROM {" . $this->table . "_category} WHERE id=%d", $_POST["cat_id"]);
		}
		elseif ($this->config_module["category"])
		{
			if ($_POST["parent_id"])
			{
				$this->site_id = DB::query_result("SELECT site_id FROM {" . $this->table . "} WHERE id=%d", $_POST["parent_id"]);
			}
			else
			{
				$this->site_id = $_POST["site_id"];
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Псевдоссылка"
	 *
	 * @return boolean
	 */
	protected function save_rewrite_module()
	{
		if (!$this->site_id)
		{
			$this->get_site_id();
		}
		if ($this->site_id != $this->oldrow["site_id"])
		{
			DB::query("UPDATE {rewrite} SET site_id=%d WHERE module_name='%s' AND " . ( $this->config_module["category"] ? "cat" : "element" ) . "_id=%d", $this->site_id, $this->diafan->module, $this->diafan->save);
			if ($this->config_module["category"])
			{
				$child = $this->diafan->get_children($this->diafan->save, $this->table);
				DB::query("UPDATE {rewrite} SET site_id=%d WHERE module_name='%s' AND cat_id IN (%s)", $this->site_id, $this->diafan->module, implode(",", $child));
				DB::query("UPDATE {" . str_replace("_category", "", $this->table) . "} SET site_id=%d WHERE cat_id IN (%s)", $this->site_id, implode(",", $child));
				DB::query("UPDATE {rewrite} SET site_id=%d WHERE module_name='%s' AND element_id IN" . " (SELECT id FROM {" . str_replace("_category", "", $this->table) . "} WHERE cat_id IN (%s))", $this->site_id, $this->diafan->module, $this->diafan->module, implode(",", $child));
			}
		}
		if (!$_POST["rewrite"] && !ROUTE_AUTO_MODULE)
		{

			DB::query("DELETE FROM {rewrite} WHERE module_name='%s' AND " . ( $this->config_module["category"] ? "cat" : "element" ) . "_id=%d", $this->diafan->module, $this->diafan->save);
			return TRUE;
		}
		if (!$this->diafan->savenew)
		{
			$row = DB::fetch_array(DB::query("SELECT rewrite, id FROM {rewrite} WHERE module_name='%s' AND " . ( $this->config_module["category"] ? "cat" : "element" ) . "_id=%d LIMIT 1", $this->diafan->module, $this->diafan->save));
		}
		$rewrite = !empty( $row["rewrite"] ) ? $row["rewrite"] : '';
		if (!empty( $row["id"] ))
		{
			DB::query("DELETE FROM {rewrite} WHERE module_name='%s' AND " . ( $this->config_module["category"] ? "cat" : "element" ) . "_id=%d AND id<>%d", $this->diafan->module, $this->diafan->save, $row["id"]);
		}
		if ($_POST["rewrite"] && $_POST["rewrite"] == $rewrite)
		{
			return FALSE;
		}
		$rewrite_site = '';
		if (!$_POST["rewrite"] && ROUTE_AUTO_MODULE)
		{
			$this->generate_rewrite();
			$rewrite = '';
		}
		if (!$rewrite)
		{
			if ($this->config_module["element"] && !empty( $_POST["cat_id"] ))
			{
				if (!$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='%s' AND cat_id=%d", $this->diafan->module, $_POST["cat_id"]))
				{
					$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d", $this->site_id);
				}
			}
			elseif ($this->config_module["category"])
			{
				if (!$_POST["parent_id"] || !$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='%s' AND cat_id=%d", $this->diafan->module, $_POST["parent_id"]))
				{
					$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d", $_POST["site_id"]);
				}
			}
			else
			{
				$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d", $this->site_id);
			}
		}
		//$new_rewrite = ($rewrite_site ? $rewrite_site.'/' : '').$this->diafan->translit($_POST["rewrite"]);
		$new_rewrite = ( $rewrite_site ? $rewrite_site . '/' : '' ) . $_POST["rewrite"];
		if (!empty( $row["id"] ))
		{
			DB::query("UPDATE {rewrite} SET rewrite='%h' WHERE id=%d", $new_rewrite, $row["id"]);
			$id = $row["id"];
		}
		else
		{
			DB::query("INSERT INTO {rewrite} (rewrite, module_name, " . ( $this->config_module["category"] ? "cat" : "element" ) . "_id, site_id) VALUES ('%h', '%s', %d, %d)", $new_rewrite, $this->diafan->module, $this->diafan->save, $this->site_id);
			$id = DB::last_id("rewrite");
		}
		if (DB::query_result("SELECT COUNT(*) FROM {rewrite} WHERE rewrite='%h' AND id<>%d", $new_rewrite, $id))
		{
			$new_rewrite .= $id;
			DB::query("UPDATE {rewrite} SET rewrite='%h' WHERE id='%d'", $new_rewrite, $id);
		}
		return TRUE;
	}

	/**
	 * Генерирует псевдоссылку
	 *
	 * @return boolean true
	 */
	protected function generate_rewrite()
	{
		//если псевдоссылка не задана, она генерируется из имени элемента/категории
		if ($this->diafan->rewrite == "faq")
		{
			$_POST["rewrite"] = strip_tags($_POST["anons"]);
		}
		else
		{
			$_POST["rewrite"] = $_POST["name"];
		}
		$route_method = DB::query_result("SELECT value FROM {config} WHERE module_name='route' AND name='method'");
		switch ($route_method)
		{
			//перевод на английский
			case 2:
				$google_api_key = DB::query_result("SELECT value FROM {config} WHERE module_name='route' AND name='google_api_key'");
				$resp = file_get_contents('https://www.googleapis.com/language/translate/v2?key=' . $google_api_key . '&source=ru&target=en&callback=translateText&q=' . urlencode($_POST["rewrite"]));
				echo $resp;
				exit;
				if (preg_match('/"translatedText"([^"]+)"([^"]+)"/', $resp, $match))
				{
					$_POST["rewrite"] = $match[2];
				}
				$_POST["rewrite"] = preg_replace('/[^A-Za-z0-9-_]+/', '', str_replace(' ', '-', $_POST["rewrite"]));
				//берутся первые 50 символов
				$_POST["rewrite"] = strtolower(substr($_POST["rewrite"], 0, 50));
				break;
			//русская кириллица
			case 3:
				$_POST["rewrite"] = preg_replace('/[^A-Za-zа-яА-Я0-9-_]+/', '', str_replace(' ', '-', $_POST["rewrite"]));
				//берутся первые 50 символов
				$_POST["rewrite"] = utf::strtolower(utf::substr($_POST["rewrite"], 0, 50));
				break;
			//транскрипция
			default:
				$route_translit_array = DB::query_result("SELECT value FROM {config} WHERE module_name='route' AND name='translit_array'");
				if ($route_translit_array)
				{
					list( $ru, $eng ) = explode('````', $route_translit_array, 2);
					$ru_arr = explode('|', $ru);
					$eng_arr = explode('|', $eng);
					$_POST["rewrite"] = str_replace($ru_arr, $eng_arr, $_POST["rewrite"]);
				}
				else
				{
					$_POST["rewrite"] = $this->diafan->translit($_POST["rewrite"]);
				}
				$_POST["rewrite"] = preg_replace('/[^A-Za-z0-9-_]+/', '', $_POST["rewrite"]);
				//берутся первые 50 символов
				$_POST["rewrite"] = strtolower(substr($_POST["rewrite"], 0, 50));
				break;
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Родитель"
	 *
	 * @return boolean
	 */
	protected function save_parent_id_module()
	{
		$_POST['parent_id']=(int)preg_replace("/\D/",'',$_POST['parent_id']);
		$this->parent_id=$_POST['parent_id'];

		$this->query[] = "parent_id='%d'";
		$this->value[] = $_POST["parent_id"];

		// если пункт новый, просто добавляем всех его родителей в table_parents и увеличиваем у родителей количество детей
		if ($this->diafan->savenew)
		{
			if ($_POST["parent_id"])
			{
				$parents = $this->diafan->get_parents($_POST["parent_id"], $this->table);
				$parents[] = $_POST["parent_id"];
				foreach ($parents as $parent_id)
				{
					DB::query("UPDATE {" . $this->table . "} SET count_children=count_children+1 WHERE id=%d", $parent_id);
					DB::query("INSERT INTO {" . $this->table . "_parents} (element_id, parent_id) VALUES (%d, %d)", $this->diafan->save, $parent_id);
				}
			}
			return TRUE;
		}
		// если родитель не изменился, уходим
		if ($this->oldrow["parent_id"] == $_POST["parent_id"])
		{
			return TRUE;
		}

		$children = $this->diafan->get_children($this->diafan->save, $this->table);
		$children[] = $this->diafan->save;
		$count_children = count($children);

		// если родитель был, у текущего элемента и его детей удаляем всех старых родителей, вышего текущего элемента
		// у старых родителей выше текущего элемента уменьшаем количество детей
		if ($this->oldrow["parent_id"])
		{
			$old_parents = $this->diafan->get_parents($this->diafan->save, $this->table);
			foreach ($old_parents as $parent_id)
			{
				DB::query("DELETE FROM {" . $this->table . "_parents} WHERE element_id IN (%h) AND parent_id=%d", implode(",", $children), $parent_id);
				DB::query("UPDATE {" . $this->table . "} SET count_children=count_children-%d WHERE id=%d", $count_children, $parent_id);
			}
		}
		// если новый родитель задан, то текущему элементу и его детям прибавляем новых родителей и увеличиваем у родителей количество детей
		if ($_POST["parent_id"])
		{
			$parents = $this->diafan->get_parents($_POST["parent_id"], $this->table);
			$parents[] = $_POST["parent_id"];
			foreach ($parents as $parent_id)
			{
				DB::query("UPDATE {" . $this->table . "} SET count_children=count_children+%d WHERE id=%d", $count_children, $parent_id);
				foreach ($children as $child)
				{
					DB::query("INSERT INTO {" . $this->table . "_parents} (element_id, parent_id) VALUES (%d, %d)", $child, $parent_id);
				}
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Время редактирования"
	 *
	 * @return boolean TRUE
	 */
	protected function save_timeedit_module()
	{
		$this->query[] = "timeedit='%s'";
		$this->value[] = time();
		return TRUE;
	}

	/**
	 * Сохранение поля "Время редактирования"
	 *
	 * @return boolean TRUE
	 */
	protected function save_act_module()
	{
		$lang = in_array("act", $this->language_names) ? _LANG : '';

		if ($this->config_module["parent"] && $this->oldrow['act'] != $_POST["act"])
		{
			$ids = $this->diafan->get_children($this->diafan->save, $this->table, array (), FALSE);
		}
		$ids[] = $this->diafan->save;
		if ($ids)
		{
			DB::query("UPDATE {" . $this->table . "} SET act" . $lang . "='" . ( !empty( $_POST["act"] ) ? "1" . ( $this->is_variable("timeedit") ? "', timeedit='" . time() : '' ) : '0' ) . "' WHERE id IN (" . implode(',', $ids) . ")");
		}
		if ($this->config_module['category'])
		{
			DB::query("UPDATE {" . str_replace('_category', '', $this->table) . "} SET act" . $lang . "='" . ( !empty( $_POST["act"] ) ? "1" . ( $this->is_variable("timeedit") ? "', timeedit='" . time() : '' ) : '0' ) . "' WHERE cat_id IN (" . implode(',', $ids) . ")");
		}
		if ($dirr = @opendir(ABSOLUTE_PATH . 'modules'))
		{
			while (( $module = readdir($dirr) ) !== FALSE)
			{
				if (file_exists(ABSOLUTE_PATH . 'modules/' . $module . '/admin/' . $module . '.admin.act.php'))
				{
					Customization::inc('modules/' . $module . '/admin/' . $module . '.admin.act.php');
					$func = 'act';
					$class = ucfirst($module) . '_admin_act';
					if (method_exists($class, $func))
					{
						eval( $class . '::' . $func . '($this, $this->table, $ids, ' . ( !empty( $_POST["act"] ) ? 1 : 0 ) . ');' );
					}
				}
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Доступ"
	 *
	 * @return boolean TRUE
	 */
	protected function save_access_module()
	{
		if ($this->config_module['category'])
		{
			$cat_id = $this->diafan->save;
			$element_id = 0;
		}
		else
		{
			$cat_id = 0;
			$element_id = $this->diafan->save;
		}

		$roles = array ();
		$old_roles = array ();
		$new_roles = array ();

		$result = DB::query("SELECT id FROM {users_role} WHERE trash='0'");
		while ($row = DB::fetch_array($result))
		{
			$roles[] = $row['id'];
		}
		if ($this->oldrow['access'])
		{
			$result = DB::query("SELECT role_id FROM {access} WHERE element_id=%d AND cat_id=%d AND module_name='%s'", $element_id, $cat_id, $this->diafan->module);
			while ($row = DB::fetch_array($result))
			{
				$old_roles[] = $row['role_id'];
			}
		}

		foreach ($_POST['access_role'] as $role_id)
		{
			$new_roles[] = intval($role_id);
		}
		// отмечены все роли
		if (empty( $_POST["access"]))
		{
			$new_roles = array ();
		}

		// изменений нет
		if (!array_diff($new_roles, $old_roles) && !array_diff($old_roles, $new_roles))
		{
			return TRUE;
		}
		$this->query[] = "access='%d'";
		$this->value[] = $new_roles ? 1 : 0;
		// Роли, которым дан доступ
		$diff_new_roles = array_diff($new_roles, $old_roles);
		foreach ($diff_new_roles as $role_id)
		{
			DB::query("INSERT INTO {access} (element_id, cat_id, module_name, role_id) VALUES (%d, %d, '%s', %d)", $element_id, $cat_id, $this->diafan->module, $role_id);
		}
		// Роли, для которых теперь нет доступа
		$diff_old_roles = array_diff($old_roles, $new_roles);
		if ($diff_old_roles)
		{
			DB::query("DELETE FROM {access} WHERE element_id=%d AND cat_id=%d AND module_name='%s' AND role_id IN (%s)", $element_id, $cat_id, $this->diafan->module, implode(",", $diff_old_roles));
		}

		// для категории можем изменить доступ для вложенных категорий и элементов
		if ($this->config_module['category_rel'])
		{
			$result = DB::query("SELECT id, access FROM {" . str_replace('_category', '', $this->table) . "} WHERE cat_id=%d", $cat_id);
			while ($row = DB::fetch_array($result))
			{
				$old_roles_el = array ();
				if ($row["access"])
				{
					$result_a = DB::query("SELECT role_id FROM {access} WHERE element_id=%d AND module_name='%s'", $row["id"], $this->diafan->module);
					while ($row_a = DB::fetch_array($result_a))
					{
						$old_roles_el[] = $row_a['role_id'];
					}
				}
				// Если доступ полностью совпадает, то изменения синхронные
				if (!array_diff($old_roles_el, $old_roles) && !array_diff($old_roles, $old_roles_el))
				{
					foreach ($diff_new_roles as $role_id)
					{
						DB::query("INSERT INTO {access} (element_id, module_name, role_id) VALUES (%d, '%s', %d)", $row["id"], $this->diafan->module, $role_id);
					}
					if ($diff_old_roles)
					{
						DB::query("DELETE FROM {access} WHERE element_id=%d AND module_name='%s' AND role_id IN (%s)", $row["id"], $this->diafan->module, implode(",", $diff_old_roles));
					}
					if (!$new_roles && $row["access"] || $new_roles && !$row["access"])
					{
						DB::query("UPDATE {" . str_replace('_category', '', $this->table) . "} SET access='%d' WHERE id=%d", $row["access"] ? 0 : 1, $row["id"]);
					}
				}
				else
				{
					$diff_old_roles_el = array_diff($old_roles_el, $new_roles);
					foreach ($diff_new_roles_el as $role_id)
					{
						DB::query("DELETE FROM {access} WHERE element_id=%d AND module_name='%s' AND role_id=%d", $row["id"], $this->diafan->module, $role_id);
					}
				}
			}

			$children = $this->diafan->get_children($cat_id, $this->table);
			if ($children)
			{
				$result = DB::query("SELECT id, access FROM {" . $this->table . "} WHERE id IN (%s)", implode(",", $children));
				while ($row = DB::fetch_array($result))
				{
					$old_roles_el = array ();
					if ($row["access"])
					{
						$result_a = DB::query("SELECT role_id FROM {access} WHERE cat_id=%d AND module_name='%s'", $row["id"], $this->diafan->module);
						while ($row_a = DB::fetch_array($result_a))
						{
							$old_roles_el[] = $row_a['role_id'];
						}
					}
					// Если доступ полностью совпадает, то изменения синхронные
					if (!array_diff($old_roles_el, $old_roles) && !array_diff($old_roles, $old_roles_el))
					{
						foreach ($diff_new_roles as $role_id)
						{
							DB::query("INSERT INTO {access} (cat_id, module_name, role_id) VALUES (%d, '%s', %d)", $row["id"], $this->diafan->module, $role_id);
						}
						if ($diff_old_roles)
						{
							DB::query("DELETE FROM {access} WHERE cat_id=%d AND module_name='%s' AND role_id IN (%s)", $row["id"], $this->diafan->module, implode(",", $diff_old_roles));
						}
						if (!$new_roles && $row["access"] || $new_roles && !$row["access"])
						{
							DB::query("UPDATE {" . $this->table . "} SET access='%d' WHERE id=%d", $row["access"] ? 0 : 1, $row["id"]);
						}
					}
					else
					{

						$diff_old_roles_el = array_diff($old_roles_el, $new_roles);
						foreach ($diff_new_roles_el as $role_id)
						{
							DB::query("DELETE FROM {access} WHERE cat_id=%d AND module_name='%s' AND role_id=%d", $row["id"], $this->diafan->module, $role_id);
						}
					}
				}
			}
		}

		return TRUE;
	}

	/**
	 * Сохранение поля "Не показывать на карте сайта"
	 *
	 * @return boolean TRUE
	 */
	protected function save_map_no_show_module()
	{
		if ($this->config_module["parent"])
		{
			$ids = $this->diafan->get_children($this->diafan->save, $this->table, array (), FALSE);
		}
		$ids[] = $this->diafan->save;
		if ($ids)
		{
			DB::query("UPDATE {" . $this->table . "} SET map_no_show='" . ( !empty( $_POST["map_no_show"] ) ? "1" : '0' ) . "' WHERE id IN (" . implode(',', $ids) . ")");
		}
		if ($this->config_module['category'])
		{
			DB::query("UPDATE {" . str_replace('_category', '', $this->table) . "} SET map_no_show='" . ( !empty( $_POST["map_no_show"] ) ? "1" : '0' ) . "' WHERE cat_id IN (" . implode(',', $ids) . ")");
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Номер страницы"
	 * @return boolean TRUE
	 */
	protected function save_number_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Шаблон страницы для разных ситуаций"
	 * @return boolean true
	 */
	protected function save_themes_module_config()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Доступ к файлам только для администратора"
     * 
	 * @return boolean true
	 */
	protected function save_attachments_access_admin_module_config()
	{
		$this->query[] = "attachments_access_admin='%d'";
		$this->value[] = 1;

		return TRUE;
	}
}