<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)) . 'includes/404.php';
}

/**
 * Dev
 *
 * Класс для работы в режиме разработки
 */
class Dev
{
	public $errors = array ();
	/**
	 * @var integer время начала работы скриптов
	 */
	private static $timestart;

	public function __construct()
	{
		// регистрация ошибок
		set_error_handler(array ( $this, 'other_error_catcher' ));

		// перехват критических ошибок
		register_shutdown_function(array ( $this, 'fatal_error_catcher' ));

		Customization::inc('includes/gzip.php');
		Gzip::init_gzip();
	}

	/**
	 * Разрешает/запрещает вывод ошибок
	 *
	 * @return boolean
	 */
	public static function set_error()
	{
		if (MOD_DEVELOPER)
		{
			ini_set('display_errors', 'on');
			error_reporting(E_ALL | E_STRICT);

			if (function_exists("xdebug_disable"))
			{
				xdebug_disable();
			}
		}
		else
		{
			ini_set('display_errors', 'off');
			error_reporting(0);
		}
		return true;
	}

	public function other_error_catcher($errno, $errstr)
	{
		if(!MOD_DEVELOPER) return false;

		$backtrace = debug_backtrace();

		if (isset( $backtrace[0]['file'] ) && preg_match('/core\.php|database\.php/', $backtrace[0]['file']))
		{
			return false;
		}

		if (isset( $backtrace[0]['file'] ) && isset( $backtrace[0]['line'] ))
		{
			$errno = $backtrace[0]['file'] . ':' . $backtrace[0]['line'];
		}
		$this->errors[] = array ( $errno, $errstr, $this->backtraceToString($backtrace) );

        $c=count($this->errors);
        echo '<a href="#error'.$c.'" style="color:red">[ERROR#'.$c.']</a>';

		return true;
	}

	private function template($result)
	{
		ob_end_clean();
		Gzip::init_gzip();


		?>
	<html>
	<head>
		<title>diafan.CMS <?php echo $result['title']?></title>
		<meta http-equiv="Content-Type" content="text/html;  charset=utf-8">
	</head>
	<body bgcolor="#FFFFFF" text="#000000" topmargin="100">
	<center>
		<table width="550" border="0" cellpadding="3" cellspacing="0">
			<tr>
				<td align="right">
					<a href="http://cms.diafan.ru/" target="_blank"><img src="http://www.diafan.ru/logo.gif" border="0" vspace="5"></a>
				</td>
				<td>
					<font face="Verdana, Arial, Helvetica, sans-serif" size="2">
						<font color="red">
							<?php echo $result['error']['message']?></font></b><br>
						<?php echo $result['error']['file']?>:<?php echo $result['error']['line']?>
					</font>
				</td>
			</tr>
		</table>
	</center>
	</body>
	</html>
	<?php
	}

	public function fatal_error_catcher()
	{
		$error = error_get_last();

		if (isset( $error ) && ( $error['type'] & ( E_ERROR | E_PARSE | E_COMPILE_ERROR | E_CORE_ERROR ) ))
		{
			header("HTTP/1.0 500 Internal Server Error");


			$result = array ( 'title'=> "Fatal Error",
							  'error'=> $error );
			$this->template($result);

		}
		else
		{
			if (MOD_DEVELOPER && count($this->errors))
			{
				echo '<div class="diafan_errors"><table>';
                $i=1;
				foreach ($this->errors as $e)
				{
					echo '<tr><td ' . ( !empty( $e[2] ) ? 'class="calls"' : '' ) . '>' . $e[1] . '<div>' . $e[2] . '</div></td><td class="file"><a name="error'.$i++.'">' . $e[0] . '</a></td></tr>';
				}
				echo '</table></div>';
			}
		}


		Gzip::do_gzip();
	}

	/**
	 * Активирует профилирование запросов, если это разрешено в параметрах
	 *
	 * @return boolean
	 */
	public static function set_profiling()
	{
		if (!defined('MOD_DEVELOPER_PROFILING') || !MOD_DEVELOPER_PROFILING)
		{
			return false;
		}

		DB::query("SET profiling_history_size=100;");
		DB::query("SET profiling=1;");

		$mtime = microtime();
		$mtime = explode(" ", $mtime);
		self::$timestart = $mtime[1] + $mtime[0];

		return true;
	}

	/**
	 * Профилирование запросов
	 *
	 * @return boolean
	 */
	public static function get_profiling()
	{
		if (!defined('MOD_DEVELOPER_PROFILING') || !MOD_DEVELOPER_PROFILING)
		{
			return false;
		}

		echo '<br><br><table border="1"><tr><td>Query_ID</td><td>Duration</td><td>Query</td></tr>';
		$result = DB::query("SHOW PROFILES");
		$summ = 0;
		while ($row = DB::fetch_array($result))
		{
			echo '<tr><td>' . $row["Query_ID"] . '</td><td>' . $row["Duration"] . '</td><td>' . $row["Query"] . '</td></tr>';
			$summ += $row["Duration"];
		}
		echo '<tr><td></td><td>' . $summ . '</td><td></td></tr></table><br><br>';

		/*
		echo '<br><br><table border="1"><tr><td>Status</td><td>Duration</td></tr>';
		$result = DB::query("SHOW PROFILE FOR QUERY 75");
		$summ = 0;
		while ($row = DB::fetch_array($result))
		{
		echo '<tr><td>'.$row["Status"]
		.'</td><td>'.$row["Duration"]
		.'</td></tr>';
		$summ += $row["Duration"];
		}
		echo '<tr><td></td><td>'.$summ.'</td></tr></table><br><br>';
		*/

		$mtime = microtime();
		$mtime = explode(" ", $mtime);
		$mtime = $mtime[1] + $mtime[0];
		$totaltime = ( $mtime - self::$timestart );

		printf("Страница сгенерирована за %f секунд", $totaltime);
		return true;
	}

	/**
	 * Analog for debug_print_backtrace(), but returns string.
	 *
	 * @return string
	 */
	public static function backtraceToString($backtrace)
	{
		// Iterate backtrace
		$calls = array ();
		foreach ($backtrace as $i => $call)
		{

			if ($i == 0)
			{
				continue;
			}

			if (!isset( $call['file'] ))
			{
				$call['file'] = '(null)';
			}

			if (!isset( $call['line'] ))
			{
				$call['line'] = '0';
			}
			$location = $call['file'] . ':' . $call['line'];
			$function = ( isset( $call['class'] ) ) ? $call['class'] . ( isset( $call['type'] ) ? $call['type'] : '.' ) . $call['function'] : $call['function'];

			$params = '';
			if (isset( $call['args'] ) && is_array($call['args']))
			{
				$args = array ();
				foreach ($call['args'] as $arg)
				{
					if (is_array($arg))
					{
						$args[] = "Array(...)";
					}
					elseif (is_object($arg))
					{
						$args[] = get_class($arg);
					}
					else
					{
						$args[] = $arg;
					}
				}
				$params = htmlspecialchars(implode(', ', $args));
			}

			$calls[] = sprintf('#%d  %s(%s) called at [%s]', $i, $function, $params, $location);
		}

		return implode("<br>\n", $calls) . "\n";
	}

}


