<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Image
 * Набор функций для работы с изображениями
 */
class Image
{
	/**
	 * Изменяет размеры изображения
	 * @param string $src_image путь к файлу
	 * @param integer $dest_width новая ширина изображения
	 * @param integer $dest_height новая высота изображения
	 * @param integer $quality качество изображения
	 * @param  boolean $max изменять по максимальной стороне
	 */
	public static function resize($src_image, $dest_width, $dest_height, $quality = 80, $max = false)
	{ 
		if (! $quality)
		{
			$quality = 80;
		}
		if (! $src_image) 
			return false;
	
		$dest_image = $src_image;
	
		$info = getImageSize($src_image); 
		switch ($info[2])
		{ 
			case 1: 
				$src_img = imageCreateFromGIF($src_image); 
				break; 
			case 2: 
				$src_img = imageCreateFromJPEG($src_image); 
				break; 
			case 3: 
				$src_img = imageCreateFromPNG($src_image); 
				break; 
			default: 
				return false; 
		}

		$src_width = imageSX ($src_img); 
		$src_height = imageSY ($src_img); 
		$mc1 = $dest_width / $src_width; 
		$mc2 = $dest_height / $src_height;

		if ($max)
		{
			$k = max($mc1, $mc2);
		}
		else
		{
			$k = min($mc1, $mc2);
		}

		$dest_width = round($src_width * $k); 
		$dest_height = round($src_height * $k); 
		$dst_img = imageCreateTrueColor($dest_width, $dest_height); 
		imageCopyResampled($dst_img, $src_img, 0, 0, 0, 0, $dest_width, $dest_height, $src_width, $src_height); 
		switch($info[2])
		{ 
			case 1: 
				imageGIF($dst_img, $dest_image);
				break; 
			case 2: 
				imageJPEG($dst_img, $dest_image, $quality); 
				break; 
			case 3:
				imagePNG($dst_img, $dest_image);
				break; 
			default: 
				return false; 
		}
		imageDestroy($src_img); 
		imageDestroy($dst_img);

		return true; 
	}

	/**
	 * Добавляет водяной знак на изображение
	 * @param string $original путь к файлу
	 * @param string $watermark путь к водяному знаку
	 * @param string $placement расположение знака
	 * @return boolean
	 */
	public static function watermark($original, $watermark, $placement)
	{
		if (! $placement)
		{
			$placement = 'bottom=5,right=5';
		}
		$original_url =  $original;
		$original = urldecode($original);
		$info_o   = @getImageSize($original); 

		if (! $info_o) 
			return false;

		$info_w = @getImageSize($watermark); 
		if (! $info_w) 
			return false;

		list($vertical,  $horizontal) = explode(',', $placement,2); 
		list($vertical,  $sy)         = explode('=', trim($vertical), 2); 
		list($horizontal, $sx)        = explode('=', trim($horizontal), 2); 

		switch(trim($vertical))
		{ 
			case 'bottom': 
				$y = $info_o[1] - $info_w[1] - (int)$sy; 
				break; 
			case 'middle': 
				$y = ceil($info_o[1]/2) - ceil($info_w[1]/2) + (int)$sy; 
				break; 
			default: 
				$y = (int)$sy; 
				break; 
		} 
		switch (trim($horizontal))
		{ 
			case 'right': 
				$x = $info_o[0] - $info_w[0] - (int)$sx; 
				break; 
			case 'center': 
				$x = ceil($info_o[0]/2) - ceil($info_w[0]/2) + (int)$sx; 
				break; 
			default: 
				$x = (int)$sx; 
				break; 
		} 

		$original  = @imageCreateFromString(file_get_contents($original)); 
		$watermark = @imageCreateFromString(file_get_contents($watermark)); 
		$out       = imageCreateTrueColor($info_o[0],$info_o[1]); 

		imageCopy($out, $original, 0, 0, 0, 0, $info_o[0], $info_o[1]); 

		if ($info_o[0] > 10 && $info_o[1] > 10)
		{
			imageCopy($out, $watermark, $x, $y, 0, 0, $info_w[0], $info_w[1]);
		} 
		switch ($info_o[2])
		{ 
			case 1: 
				imageGIF($out, $original_url); 
				break; 
			case 2: 
				imageJPEG($out, $original_url, 90); 
				break; 
			case 3: 
				imagePNG($out, $original_url); 
				break; 
		}

		imageDestroy($out); 
		imageDestroy($original); 
		imageDestroy($watermark); 

		return true; 
	}
}