<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
    include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * User
 * 
 * Работа с пользователями в пользовательской части
 */
class User extends Core
{
    /**
     * @var integer идентификатор пользователя
     */
    public $id = 0;

	/**
     * @var integer тип пользователя
     */
    public $role_id = 0;

    /**
     * @var string логин пользователя
     */
    public $name;

    /**
     * @var string ФИО пользователя
     */
    public $fio;

    /**
     * @var string e-mail пользователя
     */
    public $mail;

    /**
     * @var integer дата регистрации текущего пользователя
     */
    public $created;

    /**
     * @var integer язык интерфейса, который выбрал текущий пользователь
     */
    public $language;

    /**
     * @var string данные, хранящиеся в сессии
     */
    public $session;

    /**
     * @var boolean пользователь проверен по идентификационному хэшу
     */
    public $checked;

    /**
     * @var boolean пользователь является администратором
     */
    public $admin;

    /**
     * @var boolean пользователь использует визуальный редактор
     */
    public $htmleditor;

	/**
	 * @var boolean пользователь использует визуальный редактор
	 */
	public $background;

    /**
     * @var array права доступа администратора
     */
    public $roles = array();

    /**
     * @var array права доступа для модулей
     */
    public $module_roles = array();

    /**
     * @var string ошибка авторизации
     */
    public $errauth;

    /**
     * @var array характеристики пользователя
     */
    private $fields = array('id', 'name', 'fio', 'mail', 'created', 'role_id', 'language', 'htmleditor', 'background');

    /**
     * Конструктор класса
     * 
     * @return void
     */
    public function __construct()
    {
	
    }

    /**
     * Определяет текущего пользователя
     * 
     * @param object $user данные о текущем пользователе
     * @return boolean TRUE
     */
    public function set($user)
    {
		foreach ($this->fields as $field)
		{
			if (!empty($user->$field))
			{
				$this->$field = $user->$field;
			}
		}
		if (!empty($user->session))
		{
			$this->session = $user->session;
		}
		if (!empty($_POST["check_hash_user"]) && $this->id)
		{
			if ($id = DB::query_result("SELECT id FROM {sessions_hash} WHERE user_id=%d AND created>%d AND hash='%h'", $this->id, time() - 7200, $_POST["check_hash_user"]))
			{
				DB::query("DELETE FROM {sessions_hash} WHERE id=%d", $id);
				$this->checked = TRUE;
			}
		}
		else
		{
			$this->checked = FALSE;
		}
		return TRUE;
    }

    /**
     * Генерируем идентификационный пользовательский хэш
     * 
     * @return string
     */
    public function get_hash()
    {
		if ($this->id)
		{
			$pass = DB::query_result("SELECT password FROM {users} WHERE id=%d", $this->id);
			$hash = md5(substr($pass, mt_rand(0, 32), mt_rand(0, 32)).mt_rand(23, 567).substr($pass, mt_rand(0, 32), mt_rand(0, 32)));

			DB::query("INSERT INTO {sessions_hash} (user_id, created, hash) VALUES (%d, %d, '%h')", $this->id, time(), $hash);
			DB::query("DELETE FROM {sessions_hash} WHERE created<%d", time() - 7200);
			return $hash;
		}
		return '';
    }

    /**
     * Определяет, что текущий пользователь не авторизован
     * 
     * @param string $session данные, записанные в сессии
     * @return boolean TRUE
     */
    public function anonymous($session = '')
    {
		$this->session = $session;
		return TRUE;
    }

    /**
     * Очищает информацию о текущем пользователе
     * 
     * @return boolean TRUE
     */
    public function logout()
    {
		$this->anonymous();
		session_destroy();
		$this->redirect(BASE_PATH);
		return TRUE;
    }

    /**
     * Проверяет авторизован ли пользователь
     * 
     * @return boolean
     */
    public function auth($form_values)
    {
		if (!$form_values['name'] || !$form_values['pass'])
		{
			$this->errauth = 'wrong_login_or_pass';
			return FALSE;
		}

		if (DB::query_result("SELECT name FROM {users} WHERE (trash='1' OR act='0') AND LOWER(name)=LOWER('%s')", trim($form_values['name'])))
		{
			$this->errauth = 'blocked';
			return FALSE;
		}

		if ($this->_log())
		{
			$this->errauth = 'blocked_30_min';
			return FALSE;
		}

		$result = DB::query("SELECT * FROM {users} u WHERE trash='0' AND act='1' AND LOWER(name)=LOWER('%s') AND password='%s'", trim($form_values['name']), code_password(trim($form_values['pass'])));

		if (DB::num_rows($result))
		{
			$user = DB::fetch_object($result);

			$this->set($user);

			$rew = '';
			if ($this->language)
			{
			global $diafan;
			foreach ($diafan->language_all as $l => $langarr)
			{
				if ($this->language == $langarr[0])
				{
				$rew = $langarr[1].'/';
				break;
				}
			}
			}
			$this->redirect(BASE_PATH.$rew.($_REQUEST["rewrite"] ? $_REQUEST["rewrite"]."/" : ''));
			return TRUE;
		}
		else
		{
			$this->update_log();
			$this->errauth = 'wrong_login_or_pass';
			return FALSE;
		}
    }

    /**
     * Проверяет есть ли права у пользователя на действие для модуля
     * 
	 * @param string $action действие
	 * @param string $module_name модуль
	 * @param array $roles права пользователя
	 * @param string $type часть сайта административная/пользовательская
     * @return boolean
     */
    public function roles($action, $module_name = '', $roles = array(), $type = 'admin')
    {
		if (! $roles)
		{
			$roles = $type == 'admin' ? $this->roles : $this->module_roles;
			if (empty($roles))
			{
				return FALSE;
			}
		}

		if(! empty($roles['all']))
		{
			return TRUE;
		}

		if (!$module_name)
		{
			$module_name = $this->module;
		}

		$res = FALSE;

		if (!empty($roles[$module_name]))
		{
			if ($roles[$module_name][0] == 'all' || in_array($action, $roles[$module_name]))
			{
				$res = TRUE;
			}
		}
		return $res;
    }

    /**
     * Проверяет лог авторизации для блокировки попытки подбора паролей
     * 
     * @return boolean
     */
    private function _log()
    {
		DB::query("DELETE FROM {log} WHERE created<'%d'", time());
		$ip = getenv('REMOTE_ADDR');
		$result = DB::query("SELECT `count` FROM {log} WHERE ip = '%h'", $ip);
		if (DB::num_rows($result) > 0)
		{
			$row = DB::fetch_array($result);
			if ($row['count'] > 4)
			{
				return TRUE;
			}
		}
		return FALSE;
    }

    /**
     * Обновляет лог авторизации для блокировки попытки подбора паролей
     * 
     * @return boolean
     */
    private function update_log()
    {
		$ip = getenv('REMOTE_ADDR');
		$date = time() + 1800;
		$result = DB::query("SELECT `count` FROM {log} WHERE ip = '%h'", $ip);
		if (DB::num_rows($result) > 0)
		{
			DB::query("UPDATE {log} SET `count` = `count` + 1, created = '%d' WHERE ip = '%h'", $date, $ip);
		}
		else
		{
			$info = getenv('HTTP_USER_AGENT');
			DB::query("INSERT INTO {log} (ip, created, info) VALUES ('%s', '%d', '%s')", $ip, $date, $info);
		}
		return FALSE;
    }
}