<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_ajax
 *
 * Обработка запроса при добавлении товара в корзину
 */
class Shop_ajax extends Ajax
{
	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (empty($_POST['module']) || $_POST['module'] != 'shop' || empty($_POST['action']))
		{
			return false;
		}
		
		switch($_POST['action'])
		{
			case 'buy':
				return $this->buy();

			case 'wish':
				return $this->wish();

			case 'add_coupon':
				return $this->add_coupon();

			case 'compare_goods':
				return $this->compare_goods();
		}
		return false;
	}

	/**
	 * Добавляет товар в корзину
	 * 
	 * @return boolean
	 */
	private function buy()
	{
		if (! empty($_POST['addshop']) && (! $this->diafan->configmodules('security_user') || $this->diafan->_user->id))
		{
			$_POST['addshop'] = intval($_POST['addshop']);
			$this->tag = 'shop'.$_POST['addshop'];
			
            $row = DB::fetch_array(DB::query("SELECT id, is_file FROM {shop} WHERE id=%d AND trash='0' AND [act]='1'", $_POST['addshop']));
            
            if (empty($row['id']))
			{
				$this->result["errors"][0] = 'ERROR';
				return $this->send_errors();
			}
            
            define('GOOD_IS_FILE', (bool)$row['is_file']);
           
            $count = ! empty($_POST["count"]) ? intval($_POST["count"]) : 1;
			$count = $count > 0 ? $count : 1;
            
            $params = array();

			$result = DB::query("SELECT p.[name], p.id FROM {shop_param} as p WHERE p.`type`='multiple' AND p.required='1'"
								." AND (SELECT COUNT(*) FROM {shop_param_element} WHERE element_id=%d AND param_id=p.id) > 1", $_POST['addshop']);
			while ($row_param = DB::fetch_array($result))
			{
				if (empty($_POST["param".$row_param["id"]]))
				{
					$this->result["errors"][0] = _LANG_PLEASE_SELECT.' '.$row_param["name"];
					$this->send_errors();
					return true;
				}
				else
				{
					$params[$row_param["id"]] = intval($_POST["param".$row_param["id"]]);
				}
			}
			asort($params);
			$param = '|';
			foreach ($params as $key => $value)
			{
				$param .= $key.'='.$value.'|';
			}
			$price = $this->diafan->_shop->price_get($_POST['addshop'], $params, $this->diafan->_user->id);
			if(! $price)
			{
				$this->result["errors"][0] = 'ERROR';
				return $this->send_errors();
			}
			$count_good = 0;
			if (! empty($_SESSION["cart"][$_POST['addshop']]))
			{
				foreach ($_SESSION["cart"][$_POST['addshop']] as $key => $cart)
				{
					if ($key == $param)
					{
						$count_good = $cart["count"];
					}
				}
			}
			$count_good += $count;
			if ($this->diafan->configmodules('use_count_goods', 'shop'))
			{
				if ($count_good > $price["count_goods"])
				{
					$this->result["errors"][0] = sprintf(_LANG_NOT_GOOD, $price["count_goods"]);
					$this->send_errors();
					return true;
				}
			}

			if (isset($_SESSION["cart"][$_POST['addshop']][$param]))
			{
				$_SESSION["cart"][$_POST['addshop']][$param]["count"] +=$count;
			}
			else
			{
				$_SESSION["cart"][$_POST['addshop']][$param]["count"] = $count;
				$_SESSION["cart"][$_POST['addshop']][$param]["param"] = $params;
			}
            
            if(GOOD_IS_FILE)//товар-файл => можно купить только 1 товар
            {
                if($_SESSION["cart"][$_POST['addshop']][$param]["count"] > 1)
                {
                    $good_is_file_error = TRUE;
                }
                $_SESSION["cart"][$_POST['addshop']][$param]["count"] = 1;
                $_SESSION["cart"][$_POST['addshop']][$param]["is_file"] = TRUE;
            }
			$summ  = 0;
			$count = 0;
			foreach ($_SESSION["cart"] as $good_id => $array)
			{
				foreach ($array as $param => $c)
				{					
					$summ += $price["price"] * $c["count"];
					$count += $c["count"];
				}
			}
			$_SESSION["cart_summ"]     = $summ;
			$_SESSION["cart_count"]    = $count;
			
			DB::query("UPDATE {shop} SET counter_buy=counter_buy+1 WHERE id='%d'",$_POST['addshop']);

			$this->result["errors"][0] = ! empty($good_is_file_error) ?  _LANG_GOOD_IS_FILE_ERROR :
                    sprintf(_LANG_GOOD_IN_CART, BASE_PATH_HREF.$this->diafan->_route->module("cart",true), $count_good);
            
			$cart_tpl = array("count" => $count, "summ" => $this->diafan->format_price($summ), "currency" => $this->diafan->configmodules("currency", "shop"));
            $this->result["target"] = "#show_cart";
            $this->result["data"]   = $this->diafan->_tpl->get('info', 'cart', $cart_tpl);

			return $this->send_errors();
		}
		return false;
	}

	/**
	 * Добавляет товар в лист пожеланий
	 * 
	 * @return boolean
	 */
	private function wish()
	{
		if (! empty($_POST['addshop']) && (! $this->diafan->configmodules('security_user') || $this->diafan->_user->id))
		{
			$_POST['addshop'] = intval($_POST['addshop']);
			
            $row = DB::fetch_array(DB::query("SELECT id, is_file FROM {shop} WHERE id=%d AND trash='0' AND [act]='1'", $_POST['addshop']));
            
            if (empty($row['id']))
			{
				$this->result["errors"][0] = 'ERROR';
				return $this->send_errors();
			}
           
			$count = $this->diafan->get_param($_POST, "count", 1, 2);
			$count = $count > 0 ? $count : 1;
            
            $params = array();

			$result = DB::query("SELECT p.[name], p.id FROM {shop_param} as p WHERE p.`type`='multiple' AND p.required='1'"
								." AND (SELECT COUNT(*) FROM {shop_param_element} WHERE element_id=%d AND param_id=p.id) > 1", $_POST['addshop']);
			while ($row_param = DB::fetch_array($result))
			{
				if (empty($_POST["param".$row_param["id"]]))
				{
					$this->result["errors"][0]= _LANG_PLEASE_SELECT.' '.$row_param["name"];
					$this->send_errors();
					return true;
				}
				else
				{
					$params[$row_param["id"]] = intval($_POST["param".$row_param["id"]]);
				}
			}
			asort($params);
			$param = '|';
			foreach ($params as $key => $value)
			{
				$param .= $key.'='.$value.'|';
			}
			$price = $this->diafan->_shop->price_get($_POST['addshop'], $params, $this->diafan->_user->id);
			if(! $price)
			{
				$this->result["errors"][0] = 'ERROR';
				return $this->send_errors();
			}

			if (isset($_SESSION["wishlist"][$_POST['addshop']][$param]))
			{
				$_SESSION["wishlist"][$_POST['addshop']][$param]["count"] +=$count;
			}
			else
			{
				$_SESSION["wishlist"][$_POST['addshop']][$param]["count"] = $count;
				$_SESSION["wishlist"][$_POST['addshop']][$param]["param"] = $params;
			}

			$this->result["errors"][0] = sprintf(_LANG_GOOD_ADDED_WISHLIST, BASE_PATH_HREF.$this->diafan->_route->module("wishlist",true));

			return $this->send_errors();
		}
		return false;
	}

	/**
	 * Добавляет/удаляет товар для сравнения
	 * 
	 * @return boolean
	 */
	private function compare_goods()
	{
        if(empty($_POST['shop_compare_id']) || empty($_POST['shop_compare_site_id']))
        {
            return false;
        }
        $id = $this->diafan->get_param($_POST, "shop_compare_id", 0, 2);
        $site_id = $this->diafan->get_param($_POST, "shop_compare_site_id", 0, 2);
        
        if(empty($_POST['shop_compare_add']))
        {
            if(isset($_SESSION['compare_list'][$site_id][$id]))
            {
                unset($_SESSION['compare_list'][$site_id][$id]);
            }
        }
        else
        {
            $_SESSION['compare_list'][$site_id][$id] = 1;
            $this->result['shop_compare_add'] = 1;
        }
        $this->result['shop_compare_id'] = $id;
        $this->result['shop_compare_input'] = '<input type="hidden" name="ids[]" value="'.$id.'" class="shop_compared_goods_'.$id.'">';
        if(! empty($_POST["ajax"]))
		{
			include_once ABSOLUTE_PATH.'plugins/json.php';
			echo to_json($this->result);
		}
		else
		{
			$this->diafan->redirect($_SERVER['HTTP_REFERER']);
		}
        return true;
    }

    /**
	 * Активирует купон
	 * 
	 * @return boolean
	 */
	private function add_coupon()
	{
		if(! $this->diafan->_user->id)
		{
			return false;
		}
		if(empty($_POST["coupon"]))
		{
			$this->result["errors"][0] = _LANG_SHOP_ADD_COUPON_ERROR;
			return $this->send_errors();
		}
		$discount_id = DB::query_result("SELECT id FROM {shop_discount} WHERE coupon='%h' AND user_id=0 AND trash='0' AND act='1'", $_POST["coupon"]);
		if(! $discount_id)
		{
			$this->result["errors"][0] = _LANG_SHOP_ADD_COUPON_ERROR;
		}
		else
		{
			DB::query("UPDATE {shop_discount} SET user_id=%d WHERE id=%d", $this->diafan->_user->id, $discount_id);
			$this->diafan->_shop->price_calc(0, $discount_id);
			$this->result["redirect"] = $_SERVER['HTTP_REFERER'];
		}
		return $this->send_errors();
	}
}