<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
    include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Useradmin_inc
 *
 * Подключение модуля "Редактирование из пользовательской части"
 */
class Useradmin_inc extends Model
{

    /**
     * Генерирует ссылку на форму редактирования
     *
     * @param string $text значение переменной
     * @param string $name название переменной
     * @param integer $element_id номер элемента
     * @param string $module_name модуль
     * @param integer|string $lang_id номер языка
     * @param string $type тип данных
     * @return string
     */
    public function get($text, $name, $element_id, $module_name, $lang_id = '', $type = '')
    {
		if (!$text)
		{
			return $text;
		}

		if (! USERADMIN ||  ! $this->diafan->_user->roles('edit', 'useradmin', $this->diafan->_user->module_roles) || ! $this->diafan->_user->roles('edit', $module_name))
		{
			return $text;
		}

		$result["text"] = $text;
		$result["name"] = $name;
		$result["element_id"] = $element_id;
		$result["module_name"] = $module_name;
		if ($type)
		{
			$result["type"] = $type;
		}
		else
		{
			$result["type"] = $this->type($name);
		}

		$result["lang_id"] = $lang_id;
		$text = $this->diafan->_tpl->get('get', 'useradmin', $result);
		return $text;
    }

    /**
     * Генерирует ссылку на форму языковой константы
     *
     * @param string $name название константы
     * @return string
     */
    public function get_lang($name)
    {
		if (!defined('_LANG_'.$name))
		{
			return '';
		}
		if (IS_ADMIN || ! USERADMIN || !$this->diafan->_user->roles('edit', "language"))
		{
			return constant('_LANG_'.$name);
		}
		$result["text"] = constant('_LANG_'.$name);
		$result["name"] = $name;
		$result["element_id"] = 0;
		$result["module_name"] = "language_file";
		$result["type"] = "text";
		$result["lang_id"] = _LANG;
		$text = $this->diafan->_tpl->get('get', 'useradmin', $result);
		return $text;
    }

    /**
     * Генерирует данные для формы редактирования
     *
     * @return boolean
     */
    public function edit()
    {
		if (! USERADMIN || empty($_GET["module_name"]) || empty($_GET["name"]))
		{
			include ABSOLUTE_PATH.'includes/404.php';
			return FALSE;
		}
		if ($_GET["module_name"] != "language_file" && empty($_GET["element_id"]))
		{
			include ABSOLUTE_PATH.'includes/404.php';
			return FALSE;
		}

		if ($_GET["module_name"] != "language_file" && !$this->diafan->_user->roles('edit',$_GET["module_name"]) || $_GET["module_name"] == "language_file" && !$this->diafan->_user->roles('edit',"language"))
		{
			include ABSOLUTE_PATH.'includes/404.php';
			return FALSE;
		}
		$result["name"] = $this->diafan->get_param($_GET, "name", "", 1);
		if ($_GET["module_name"] != "language_file")
		{
			$result["type"] = !empty($_GET["type"]) ? $_GET["type"] : $this->type($_GET["name"]);
			$result["lang_id"] = (!empty($_GET['lang_id']) ? (int) $this->diafan->get_param($_GET, "lang_id", "", 2):'');
			$result["element_id"] = (int) $this->diafan->get_param($_GET, "element_id", "", 2);
			$result["module_name"] = $this->diafan->get_param($_GET, "module_name", "", 1);
			$result["text"] = DB::query_result("SELECT %h".$result["lang_id"]." FROM {%h} WHERE id=%d", $result["name"], $result["module_name"], $result["element_id"]);
			$result["error"] = FALSE;
		}
		else
		{
			$result["type"] = "text";
			$result["lang_id"] = (int) $this->diafan->get_param($_GET, "lang_id", "", 2);
			$result["element_id"] = 0;
			$result["module_name"] = "language_file";
			$result["text"] = constant('_LANG_'.$result["name"]);
			$result["error"] = FALSE;
			$shortname = !$result["lang_id"] || $result["lang_id"] == 1 ? LANGUAGE_BASE : DB::title("language", $result["lang_id"], "shortname");
			if (file_exists(ABSOLUTE_PATH.'language/'.$shortname.'.php') && !is_writable(ABSOLUTE_PATH.'language/'.$shortname.'.php')
				|| !file_exists(ABSOLUTE_PATH.'language/'.$shortname.'.php') && !is_writable(ABSOLUTE_PATH.'language'))
			{
			$result["error"] = sprintf(_ADMIN_LANGUAGEFILE_NO_PERMISSION, 'language/'.$shortname.'.php');
			}
		}
		$user_id = DB::query_result("SELECT `user_id` FROM {sessions} WHERE session_id='%h'", session_id());
		$pass = DB::query_result("SELECT password FROM {users} WHERE id=%d", $user_id);
		$result["hash"] = md5(substr($pass, mt_rand(0, 32), mt_rand(0, 32)).mt_rand(23, 567).substr($pass, mt_rand(0, 32), mt_rand(0, 32)));

		DB::query("INSERT INTO {sessions_hash} (user_id, created, hash) VALUES (%d, %d, '%h')", $user_id, time(), $result["hash"]);

		header('Content-Type: text/html; charset=utf-8');
		$this->diafan->_tpl->get('edit', 'useradmin', $result);
		return TRUE;
    }

    /**
     * Возвращает тип данных по имени переменной
     *
     * @param string $name имя редактируемой переменной
     * @return string
     */
    public function type($name)
    {
		switch ($name)
		{
			case 'created':
				$type = 'date';
				break;
			case 'name':
				$type = 'text';
				break;
			default:
				$type = 'editor';
				break;
		}
		return $type;
    }
}